import { world, system } from "@minecraft/server";
import { ActionFormData, ModalFormData } from "@minecraft/server-ui";

world.afterEvents.entityDie.subscribe((event) => {
    const entity = event.deadEntity;

    if (entity && entity.typeId === "nuke:nuke_1_missile_dropping") {
        const death_nuke_pos = entity.location;
        entity.runCommandAsync(`summon nuke:mushroom ${death_nuke_pos.x} ${death_nuke_pos.y} ${death_nuke_pos.z}`);
    }
});

world.afterEvents.entitySpawn.subscribe((event) => {
    const entity = event.entity;

    if (entity && entity.typeId === "nuke:mushroom") {

        system.runInterval(() => {
            entity.triggerEvent("EXPLODE");
        }, 5);
    }
});



function getClosestNukeMissile(player, maxDistance = 200) {
    const nearbyEntities = world.getDimension("overworld").getEntities({
        location: player.location,
        maxDistance: maxDistance,
        type: "nuke:nuke_1_missile"
    });

    if (nearbyEntities.length > 0) {
        return nearbyEntities[0];
    }
    return null;
}

world.beforeEvents.itemUse.subscribe((event) => {
    const item = event.item;
    const source = event.source;

    if (source && item && item.typeId === "nuke:nukecode123") {
        const nukeMissile = getClosestNukeMissile(source);

        if (nukeMissile && !nukeMissile.hasTag("code_123")) {
            nukeMissile.addTag("code_123");
            console.log("Tag 'code_123' added to nuke.");

            if (nukeMissile.hasTag("code_123")) {
                source.runCommand("say Nuke code accepted! Missile tagged.");
                console.log("Tag confirmed on the entity.");
            } else {
                source.runCommand("say Tag 'code_123' was not applied correctly.");
                console.log("Tag application failed.");
            }
        } else if (!nukeMissile) {
            source.runCommand("say No nuke missile found.");
        } else {
            source.runCommand("say Nuke missile already tagged.");
        }
    }
});

function calculateDistance(loc1, loc2) {
    const dx = loc1.x - loc2.x;
    const dy = loc1.y - loc2.y;
    const dz = loc1.z - loc2.z;
    return Math.sqrt(dx * dx + dy * dy + dz * dz);
}

world.afterEvents.entityDie.subscribe((event) => {
    const deadEntity = event.deadEntity;

    if (deadEntity && deadEntity.typeId === "nuke:control_panel_gui") {
        const nearbyPlayers = world.getAllPlayers();
        for (const player of nearbyPlayers) {
            const distance = calculateDistance(deadEntity.location, player.location);
            if (distance < 10) {
                main(player);
                break;
            }
        }
    }
});

function main(player) {
    const form = new ActionFormData()
        .title("Control Panel")
        .body(`Hello ${player.nameTag}, what would you like to do?`)
        .button("Nuke Code\n [Click to enter nuke codes and coordinates]")
        .button("to be added in a future updates")
        .button("Add launch code to nearby nuke\n Activates nuke codes.")
        .button("Exit\n [Close out of the control panel]");

    form.show(player).then(response => {
        switch (response.selection) {
            case 0:
                Nuke(player);
                break;
            case 1:
                Map_Of_World(player);
                break;
            case 2:
                add_launch_code(player);
                break;
            case 3:
                Exit(player);
                break;
            default:
                console.log("Invalid selection");
        }
    }).catch(error => {
        console.error("Error showing form:", error);
    });
}

function Nuke(player) {
    const nukeForm = new ModalFormData()
        .title("Nuke Setup")
        .textField("Nuke Code", "Type your nuke code here")
        .textField("Coordinates", "Type coordinates where the nuke will blow up (format: x,y,z)");

    nukeForm.show(player).then(response => {
        const [nukeCode, coordinates] = response.formValues;

        if (nukeCode === "code_123") {
            const [x, y, z] = coordinates.split(',').map(Number);

            if (!isNaN(x) && !isNaN(y) && !isNaN(z)) {
                const nukeMissile = getClosestNukeMissile(player);
                if (nukeMissile && nukeMissile.hasTag("code_123")) {
                    nukeMissile.runCommandAsync("effect @s levitation 60 5 true");

                    system.runInterval(() => {
                        nukeMissile.teleport({ x, y: y + 200, z });
                        nukeMissile.triggerEvent("nuke_dropping");
                    }, 900);

                    nukeMissile.runCommand(`event entity @e[type=nuke:nuke_1_control_pannel, r=1000] nuke_launching_control_panel`);
                    player.runCommand(`say A nuke has been launched at (${x}, ${y + 200}, ${z})`);
                } else {
                    player.runCommand("say No nuke entity found with the correct code.");
                }
            } else {
                player.runCommand("say Invalid coordinates format. Please use x,y,z.");
            }
        } else {
            player.runCommand("say Invalid nuke code.");
        }
    }).catch(error => {
        console.error("Error showing nuke form:", error);
    });
}

function add_launch_code(player) {
    const form = new ModalFormData()
        .title("Add Launch Code")
        .textField("Launch Code", "Type new launch code for nearby nukes.");

    form.show(player).then(response => {
        const launchCode = response.formValues[0];

        if (launchCode === "code_123") {
            const nearbyEntities = world.getDimension("overworld").getEntities({
                location: player.location,
                maxDistance: 200,
                type: "nuke:nuke_1_missile"
            });

            let foundNuke = null;
            for (const entity of nearbyEntities) {
                if (!entity.hasTag("code_123")) {
                    foundNuke = entity;
                    break;
                }
            }

            if (foundNuke) {
                foundNuke.addTag("code_123");
                if (foundNuke.hasTag("code_123")) {
                    player.runCommand("say Nuke has been activated.");
                } else {
                    player.runCommand("say Failed to add activate to nuke.");
                }
            } else {
                player.runCommand("say No nearby nukes found or they are already active.");
            }
        } else {
            player.runCommand("say Invalid launch code.");
        }
    }).catch(error => {
        console.error("Error showing launch code form:", error);
    });
}

function Exit(player) {
    console.log("Exit button pressed by", player.nameTag);
}
